//=============================================================================
//  Unified IO - C++ interface for industrial IO cards
//  Copyright (C) 2000, UAB "BBD SOFT"  ( http://www.bbdsoft.com/ )
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
//  The author of this program may be contacted at developers@bbdsoft.com
//=============================================================================


#ifndef _GEMICROGENI_
#define _GEMICROGENI_

#ifndef _UNIIO_H
   #include "uniio.h"
#endif

#ifndef _IOCARD_
   #include "iocard.hpp"
#endif

#ifndef _PHYSMEM_
   #include "physmem.hpp"
#endif

//-----------------------------------------------------------------------------
namespace Uniio
{

class _Export_UNIIO GEMicroGENI : public IOCard
{
   typedef IOCard Inherited;

public:
   GEMicroGENI ( const unsigned inIOPortAddress
               , const unsigned long inSharedRamAddress
               );

   virtual ~GEMicroGENI ();

   unsigned baseIOAddress () const
      {return theBaseIOAddress;}

   bool isBoardOK () const;
   bool isCommOK () const;
   bool isReset () const;
   GEMicroGENI & reset (const bool inReset = true);
   GEMicroGENI & clearReset ();

protected:


   virtual IOCard & updateInputs ( const IOTrack & inTrack
                                 , long * inImage
                                 , long & inTrackState
                                 );

   virtual IOCard & updateOutputs ( const IOTrack & inTrack
                                  , const long * const inImage
                                 , long & inTrackState
                                  );


   virtual IOCard & addTrack ( IOTrack & inTrack ) ;


   enum ProtectedConstants
   {
        SetupTableOffset = 0x882
      , DeviceConfigurationTableOffset = 0x1E00
      , IOTableOffset = 0x2000
      , LockoutRequestOffset = 0xA48
      , LockoutStateOffset = 0xA49
      , pointsPerBank  = 8
      , maxBanks       = 256
      , maxStations    = 32
   };

private:
   GEMicroGENI (const GEMicroGENI &);
   GEMicroGENI & operator = (const GEMicroGENI &);

   unsigned char sizeOfIOTable() const
      {return theSizeOfIOTable;}
   void getSizeOfIOTable();

   bool stationExists(const unsigned inStation) const;

   GEMicroGENI & lockout ();
   GEMicroGENI & release ();

   void readDigitalInputs ( const IOTrack & inTrack
                          , long * inImage
                          , long & inTrackState
                          );

   void readAnalogInputs ( const IOTrack & inTrack
                         , long * inImage
                         , long & inTrackState
                         );

   void writeDigitalOutputs ( const IOTrack & inTrack
                            , const long * const inImage
                            , long & inTrackState
                            );

   void writeAnalogOutputs ( const IOTrack & inTrack
                           , const long * const inImage
                           , long & inTrackState
                           );


   PhysicalMemory theSharedMemory;
   const unsigned theBaseIOAddress;
   unsigned char theBankMasks[maxBanks];
   unsigned char theSizeOfIOTable;

}; // end GEMicroGENI class declaration


}; // end namespace

#endif