//=============================================================================
//  Unified IO - C++ interface for industrial IO cards
//  Copyright (C) 2000, UAB "BBD SOFT" ( http://www.bbdsoft.com/ )
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
//  The author of this program may be contacted at developers@bbdsoft.com
//=============================================================================


#ifndef _OPTO22AC28BASE_
#define _OPTO22AC28BASE_

#ifndef _UNIIO_H
   #include "uniio.h"
#endif

#ifndef _IOCARD_
   #include "iocard.hpp"
#endif

//-----------------------------------------------------------------------------
namespace Uniio
{

class _Export_UNIIO Opto22AC28Base : public IOCard
{
   typedef IOCard Inherited;

public:

   virtual ~Opto22AC28Base ();

   unsigned baseIOPort () const;
   unsigned resetPort () const;
   bool resetLevel () const;

protected:

   Opto22AC28Base ( const char * inName
            , const bool inResetLevel
            , const unsigned inIOPort = 0
            , const unsigned inResetPort = 0
            );

   virtual IOCard & updateInputs ( const IOTrack & inTrack
                                 , long * inImage
                                 , long & inTrackState
                                 );

   virtual IOCard & updateOutputs ( const IOTrack & inTrack
                                  , const long * const inImage
                                  , long & inTrackState
                                  );

   virtual IOCard & addTrack ( IOTrack & inTrack ) ;

   void setBaseIOPort (const unsigned inPort)
      {theBaseIOPort = inPort;}

   void setResetPort (const unsigned inPort)
      {theResetPort = inPort;}


private:
   Opto22AC28Base (Opto22AC28Base &);
   Opto22AC28Base & operator = (Opto22AC28Base &);

   enum AC28Constants
   {
       maxNumberOfBanks = 64 // One AC28 can serve max 64 banks
     , maxNumberOfPoints = 8 // Number of Pamux points in each bank
     , maxChannels = 16
     , semaphoreRegister = 0x82
     , lowerWatchdogTimer = 0x7A
     , upperWatchdogTimer = 0x7B
     , analogStatusRegister = 0x7C
     , lowerConfigRegister = 0x7E
     , upperConfigRegister = 0x7F
     , maxNumberOfRetries = 10
   };

   void readDigitalInputs ( const IOTrack & inTrack
                          , long * inImage
                          , long & inTrackState
                          );

   void readAnalogInputs ( const IOTrack & inTrack
                         , long * inImage
                         , long & inTrackState
                         );

   void writeDigitalOutputs ( const IOTrack & inTrack
                            , const long * const inImage
                            , long & inTrackState
                            );

   void writeAnalogOutputs ( const IOTrack & inTrack
                           , const long * const inImage
                           , long & inTrackState
                           );


   void configureAnalogBank ( const unsigned char inAddress
                            , const unsigned short inMask
                            );

   bool getAccess (const unsigned char inAddress);
   void releaseAccess (const unsigned char inAddress);
                               
   unsigned theBaseIOPort; 
   unsigned theResetPort;
   const bool theResetLevel; 

   unsigned char theBankMasks [maxNumberOfBanks];

}; // end Opto22AC28Base class declaration

#include "opto22ac28base.inl"

}; // end namespace

#endif