//=============================================================================
//  Unified IO - C++ interface for industrial IO cards
//  Copyright (C) 2000, UAB "BBD SOFT" ( http://www.bbdsoft.com/ )
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
//  The author of this program may be contacted at developers@bbdsoft.com
//=============================================================================


#ifndef _SST5136PFBPCI_
#define _SST5136PFBPCI_

#ifndef _IOCARD_
   #include "iocard.hpp"
#endif

#ifndef _UNIIO_H
   #include "uniio.h"
#endif

#ifndef _PHYSMEM_
   #include "physmem.hpp"
#endif


//-----------------------------------------------------------------------------
namespace Uniio
{

class _Export_UNIIO SST5136PFBPCI : public IOCard
{

typedef IOCard Inherited;

public:
   
   enum pfbBaudRate
   {
       baud9k6      = 0
     , baud19k2     = 1
     , baud93k75    = 2
     , baud187k5    = 3
     , baud500k     = 4
     , baud750k     = 5
     , baud1m5      = 6
     , baud3m       = 7
     , baud6m       = 8
     , baud12m      = 9
   };

   SST5136PFBPCI ( const char * const inFirmwarePath
                     , const unsigned long inBaudRate
                     , const unsigned inCardId = 0
                     ); 
   virtual ~SST5136PFBPCI();

   SST5136PFBPCI & setBaudRate (const unsigned char inBaudRate);
   unsigned char baudRate () const;

   SST5136PFBPCI & configure(const char * const inConfigFile);
   SST5136PFBPCI & autoconfigure();
   SST5136PFBPCI & configurationToFlash();
   SST5136PFBPCI & configurationFromFlash();
   SST5136PFBPCI & online();


protected:

   virtual IOCard & updateInputs ( const IOTrack & inTrack
                                 , long * inImage
                                 , long & inTrackState
                                 );

   virtual IOCard & updateOutputs ( const IOTrack & inTrack
                                  , const long * const inImage
                                  , long & inTrackState
                                  );

   virtual IOCard & addTrack ( IOTrack & inTrack ) ;

private:
   SST5136PFBPCI ( const SST5136PFBPCI & ); 
   SST5136PFBPCI & operator = ( const SST5136PFBPCI & ); 

   unsigned char state() const;

   unsigned char status() const;

   enum Constants
   {
       pfbMemorySize = 0x200000 // 2 Mb
     , pfbCommandOffset = 0x8000 // 2nd page
     , pointsPerBank  = 8
     , maxStations = 126
     , maxBytesPerStation = 244
     , maxPageSize = 0x4000 // 16Kb
   };

   enum ProfiUser // general PFB control structure 
   {
                            //        5136-PFB Basic Control/Status registers
        pfbCommand         = 0x8000 // uchr   Card command register                                        
      , pfbStatus          = 0x8001 // uchr   Card status register                                         
                                           
                              //        5136-PFB Card and Module Identification registers
      , pfbCardId          = 0x8002 // ushrt  5136-PFB Card ID (0xaad0)                                    
      , pfbModId           = 0x8004 // ushrt  PFBPROFI Module ID     (0xbb01)                              
      , pfbModVer          = 0x8006 // ushrt  PFBPROFI Module Version (ex. 0x0102 = 1.02)                  
                                           
                              //        5136-PFB --> Host Event and Interrupt control registers
      , pfbEvtEna          = 0x8008 // ushrt  Event enable mask                                            
      , pfbIntEna          = 0x800a // ushrt  Event interrupt enable mask                                  
      , pfbEvtHead         = 0x800c // uchr   Event queue head pointer (changed by card)                   
      , pfbEvtTail         = 0x800d // uchr   Event queue tail pointer (changed by host)                   
                                           
                              //        ASPC2 Profibus controller Basic Parameters
      , pfbStnAddr         = 0x800e // uchr   PFB Local station address                                    
      , pfbHiStnAddr       = 0x800f // uchr   highest station address on network                           
      , pfbActive          = 0x8010 // uchr   1=active station, 0=passive station                          
      , pfbBaud            = 0x8011 // uchr   network baud rate                                            
      , pfbOptions         = 0x8012 // ushrt  network options                                              
                                           
                              //        ASPC2 Profibus controller Bus Parameters (PFB will put in defaults if left unchanged)
      , pfbTokRotTime      = 0x8014 // ulng   Target token rotation time                                   
      , pfbSlotTime        = 0x8018 // ushrt  Slot time                                                    
      , pfbIdleTime1       = 0x801a // ushrt  Idle time 1                                                  
      , pfbIdleTime2       = 0x801c // ushrt  Idle time 2                                                  
      , pfbReadyTime       = 0x801e // ushrt  Ready time   (MinTsr)                                        
      , pfbGapUpdFact      = 0x8020 // uchr   Gap update factor                                            
      , pfbQuiTime         = 0x8021 // uchr   Tqui                                                         
                                           
                              //        ASPC2 Profibus controller Error Handling Parameters
      , pfbTokRetryLimit   = 0x8022 // uchr   Token retry limit                                            
      , pfbMsgRetryLimit   = 0x8023 // uchr   Message retry limit                                          
      , pfbTokErrLimit     = 0x8024 // uchr   Token error limit                                            
      , pfbRespErrLimit    = 0x8025 // uchr   Response error limit                                         
                                           
                              //        5136-PFB --> Host Trigger Queue control
      , pfbTrgHead         = 0x8026 // uchr   pfb function trigger event head pointer (changed by host)    
      , pfbTrgTail         = 0x8027 // uchr   pfb function trigger event tail pointer (changed by card)    
                                           
                              //        DP Master global status and control table pointer
      , pfbMasCntrlCfg     = 0x8028 // uchr   Global control for all master blocks                         
      , pfbMasSts          = 0x8029 // uchr   Global status for all master blocks                          
      , pfbMasCntrlPage    = 0x802a // uchr   Memory page which contains master block control table        
      , pfbMasRxPage       = 0x802b // uchr   Memory page which contains master Rx data (from slaves)      
      , pfbMasTxPage       = 0x802c // uchr   Memory page which contains master Tx data (to slaves)        
      , pfbMasCoherFlags   = 0x802d // uchr   Master tx and rx coherent data flags                         
      , pfbMasMaxIoCycTme  = 0x802e // ushrt  Maximum I/O cycle time in 10ms increments (0.01-655.35s)     
                                           
                              //        FMS global status and control table pointer 
      , pfbFmsCntrlCfg     = 0x8030 // uchr                                                                
      , pfbFmsSts          = 0x8031 // uchr   Global status for all FMS blocks             (NI)            
      , pfbFmsCrlPage      = 0x8032 // uchr   FMS Comm Rel List (CRL) Page                                 
                                           
                              //        MPI global status and control table pointer 
      , pfbMpiCntrlCfg     = 0x8033 // uchr                                                                
      , pfbMpiSts          = 0x8034 // uchr   Global status for all MPI blocks             (NI)            
      , pfbMpiCrlPage      = 0x8035 // uchr   MPI User Interface Page                                      
      , pfbMpiReqTime      = 0x8036 // uchr   Last Request Execution time                                                                  
                              //        DP Master sync and freeze support                   
      , pfbMasSyncFrzCmd   = 0x8037 // ushrt  sync and freeze command word (MSB=cmd, LSB=group select          
      , pfbMasScanEmpty    = 0x8039 // uchr   non-zero -> There are no slave in data exchange mode                     
                                           
      , Res3               = 0x803A // uchr[0x40-0x3a]   3f Reserved, DO NOT USE                                      
                                           
                              //        Diagnostic Counters and control
      , pfbInitCtrs        = 0x8040 // uchr   if non-zero PFB will init counters then set back to 0        
                                           
                              //        General Statistics
      , errLanOffline      = 0x8041 // uchr   Lan encountered errors and went into off-line state          
      , diagConf           = 0x8042 // ushrt  Total confirmations (to requests from us)     (MAS,LAY2,FMS) 
      , diagInd            = 0x8044 // ushrt  Total indications   (requests to us)          (MAS,LAY2,FMS) 
      , errNotOk           = 0x8046 // ushrt  Total Not OK confirmations and/or indications (MAS,LAY2,FMS) 
      , diagTokHldTime     = 0x8048 // ulng   Actual Instantaneous token hold time in Tbit                 
      , diagMinTokHldTime  = 0x804c // ulng   Minimum Actual token hold time in Tbit                       
                                           
                              //        DP Master Block Statistics
      , diagMasterUpdate   = 0x8050 // ushrt  Master I/O update cycles completed                           
      , errMasErr          = 0x8052 // uchr   Master->DP slave communication errors                        
      , errReConfig        = 0x8053 // uchr   Master->DP went offline and had to be reconfigured           
      , diagMasScanTime    = 0x8054 // ulng   Instantaneous master scan time in microseconds               
      , diagMasMaxScanTime = 0x8058 // ulng   Maximum master scan time in microseconds                     
                                           
                              //        DP Slave Statistics
      , diagSlaveUpdate    = 0x805c // ushrt  Slave updates                                                
      , errSlvErr          = 0x805e // uchr   Slave configuration failures                                 
      , errSlvTout         = 0x805f // uchr   Slave watchdog timeouts                                      
                                           
                              //        Layer 2 Message Statistics 
      , diagLay2MsgOk      = 0x8060 // ushrt  Layer 2 messages sent ok                                     
      , errLay2MsgNotOk    = 0x8062 // uchr   Layer 2 message errors                                       
      , errLay2MsgProcOvrn = 0x8063 // uchr   Layer 2 message over run                                     
                                           
                              //        Layer 2 SAP Statistics 
      , diagLay2SapOk      = 0x8064 // ushrt  Layer 2 SAP requests processed ok                            
      , errLay2SapNotOk    = 0x8066 // uchr   Layer 2 SAP errors                                           
      , errLay2SapTout     = 0x8067 // uchr   Layer 2 SAP update timeouts                                  
                                           
      , Res5               = 0x8068 // uchr[0x70-0x68]   6f Reserved, DO NOT USE                                      
                                           
                              //        ASPC2 Profibus controller Statistics
      , errInvReqLen       = 0x8070 // uchr   invalid request length errors                                
      , errFifo            = 0x8071 // uchr   FIFO overflow errors                                         
      , errRxOverun        = 0x8072 // uchr   receive overrun errors                                       
      , errDblTok          = 0x8073 // uchr   double token errors (bad wiring or hardware)                 
      , errRespErr         = 0x8074 // uchr   response errors (bad wiring or hardware)                     
      , errSyniErr         = 0x8075 // uchr   syni errors (bad wiring or hardware)                         
      , errNetTout         = 0x8076 // uchr   network timeout errors                                       
      , errHsa             = 0x8077 // uchr   Station higher than HSA was heard                            
      , errStn             = 0x8078 // uchr   Duplicate Station Detected                                   
      , errPasTok          = 0x8079 // uchr   Unable to Pass Token (bad wiring or hardware)                
      , errLasBad          = 0x807a // uchr   active station list invalid (bad wiring or hardware)         
                                           
      , errInternal        = 0x807b // uchr   
      , errArg             = 0x807c // uchr   
      , errEventOverun     = 0x807d // uchr   a new event occurred before the last one was cleared         
                                           
                              //        Active Station List
      , pfbAckLasChnge     = 0x807e // uchr   bit 0=1 to acknowledge station list change (card clears)     
      , pfbUpdPasv         = 0x807f // uchr   bit 0=1 to update status of passive stations (card clears)   
      , pfbActStnList      = 0x8080 // uchr[128]   ff Active Station list, 1 byte per station                   
                                         
      , pfbBinCfgPage      = 0x8100 // uchr    master binary configuration memory page (filled in by PFB)  
      , pfbBinCfgOfs       = 0x8101 // uchr    master binary config block offset (in 16k pages)            
      , pfbBinCfgLen       = 0x8102 // ushrt   master binary configuration length of data in current page  
                                         
      , pfbMc2PollTout     = 0x8104 // ushrt  -105 poll time out for reception of master class 2 commands  
      , pfbMasMinIoCycTme  = 0x8106 // ushrt   Minimum I/O cycle time in 100us increments (0-6.3535s)      
                                         
      , pfbWdTime          = 0x8108 // ushrt   Host Watchdog Time in ms. Non-Zero to enable                
      , pfbWdKick          = 0x810a // ushrt   Host Watchdog Kick card writes 1, host writes 0 to kick wdog
                                         
      , pfbMasGlbEvt       = 0x810c // uchr    dp master global event                                      
      , pfbMasNumBlks      = 0x810d // uchr    dp master - number of master blocks configured              
                                         
      , pfbMasCntrlExtFree = 0x810e // ushrt                                                               
                                         
      , pfbLocIdUsrStr     = 0x8110 // uchr[112]   -17f Text for the 4 fields to return in the ID response      
                             //         DP Slave controls
      , slvCntCfg          = 0x8180 // ushrt   Control/config for dp slave function                        
      , slvStatus          = 0x8182 // uchr    Status for dp slave function (Host Read Only)               
      , slvError           = 0x8183 // uchr    Error indication for dp slave function                      
      , slvEvent           = 0x8184 // uchr    Event Flags for dp slave function                           
      , slvDiagEvent       = 0x8185 // uchr    Diagnostic Event Flags for dp slave function                
                                         
      , Res9               = 0x8186 // uchr[2]   -187 Reserved, DO NOT USE                                    
                                         
      , slvRxDataLen       = 0x8188 // uchr    Receive data len (data from master)                         
      , slvReqRxDataLen    = 0x8189 // uchr    Receive data len requested by master                        
                                         
      , Res10              = 0x818a // uchr[2]   -18b Reserved, DO NOT USE                                    
                                         
      , slvTxDataLen       = 0x818c // uchr    Transmit data len (data to master)                          
      , slvReqTxDataLen    = 0x818d // uchr    Transmit data len requested by master                       
                                         
      , slvChkLen          = 0x818e // uchr    length of configuration check data                          
      , slvParmLen         = 0x818f // uchr    length of parameters from master                            
      , slvGlbCntrl        = 0x8190 // uchr[2]    global controls from master                                 
                                         
      , Res11              = 0x8192 // uchr[14]   -19f Reserved, DO NOT USE                                    
                                         
      , slvSts1            = 0x81a0 // uchr    status byte 1 to master                                     
      , slvSts2            = 0x81a1 // uchr    status byte 2 to master                                     
      , slvSts3            = 0x81a2 // uchr    status byte 3 to master                                     
      , slvMasStn          = 0x81a3 // uchr    Station that configured Slave                               
      , slvID_hi           = 0x81a4 // uchr    ID value to master hi byte (default 0x67)                   
      , slvID_lo           = 0x81a5 // uchr    ID value to master lo byte (default 0x15)                   
      , slvDiagLen         = 0x81a6 // uchr    Length of diagnostics to master                             
      , slvDiag            = 0x81a7 // uchr[25]   -1bf Diagnostics to master                                   
                                         
      , slvMasSts          = 0x81c0 // uchr    status byte from master                                     
      , slvWdFact1         = 0x81c1 // uchr    Watchdog factor 1 from master                               
      , slvWdFact2         = 0x81c2 // uchr    Watchdog factor 2 from master                               
      , slvReadyTime       = 0x81c3 // uchr    Response delay time (tbit) from master                      
      , slvMasID_hi        = 0x81c4 // uchr    ID value from master (hi byte)                              
      , slvMasID_lo        = 0x81c5 // uchr    ID value from master (lo byte)                              
      , slvGrpId           = 0x81c6 // uchr    Group ID value from master                                  
      , slvParm            = 0x81c7 // uchr[25]   -1df parameters from master                                  
      , slvChk             = 0x81e0 // uchr[32]   -1ff Configuration Check values from master                  
      , slvRxData          = 0x8200 // uchr[256]   -2ff Slave data received from master                         
      , slvTxData          = 0x8300 // uchr[256]   -3ff Slave data to be sent to master                         
                                         
      , pfbEventQueue      = 0x8400 // ushrt[256]  -5ff event queue                                             
      , pfbTrigQueue       = 0x8600 // ushrt[256]  -7ff trigger queue                                           
                                         
      , pfbIdReq           = 0x8800 // uchr    bit 0=1 to request ID for station in pfbIdStn (card clears) 
      , pfbIdStn           = 0x8801 // uchr    station to send ID request
      , pfbIdRspSts        = 0x8802 // uchr    if non-zero, error getting ID                               
      , pfbIdRspLen        = 0x8803 // uchr    total length of ID response                                 
                                         
      , pfbIdFldLen        = 0x8804 // uchr[4]   -807 Length of the 4 fields returned in the ID response      
      , pfbIdText          = 0x8808 // uchr[242]   -8f9 Text for the 4 fields returned in the ID response       
                                         
      , Res12              = 0x88fa // uchr[6]   -8ff Reserved, DO NOT USE                                    
      , pfbMasStsTab       = 0x8900 // ushrt[128]  -9ff Master status/xref table arranged by station number     
                                         
      , mc2CntCfg          = 0x8a00 // uchr   
      , mc2State           = 0x8a01 // uchr   
      , mc2Status          = 0x8a02 // uchr   
      , mc2Error           = 0x8a03 // uchr   
      , mc2Event           = 0x8a04 // uchr   
      , mc2RspSts          = 0x8a05 // uchr   
      , mc2TxLen           = 0x8a06 // uchr   
      , mc2RxLen           = 0x8a07 // uchr   
      , mc2PollTime        = 0x8a08 // ushrt  
      , mc2PollLimit       = 0x8a0a // ushrt  
      , mc2DstStn          = 0x8a0c // uchr   
                                         
      , Res13              = 0x8a0d // uchr[3]   -a0f Reserved, DO NOT USE                                    
                                         
      , mc2TxData          = 0x8a10 // uchr[244]   -b03
      , mc2RxData          = 0x8b04 // uchr[244]   -bf7
                                         
      , Res14              = 0x8bf8 // uchr[0xfe0-0xbf8]   -fdf Reserved, DO NOT USE                                    
                                         
                             //         Layer 2 global status and control table pointer
      , lay2Cntrl          = 0x8fe0 // ushrt   Global control for all Layer 2 SAP's and MSG's              
      , lay2Status         = 0x8fe2 // ushrt   Global status  for all Layer 2 SAP's and MSG's              
                                         
      , Res15              = 0x8fea // uchr[0x1000-0xfe4]   -fff Reserved, DO NOT USE                                    
                                    
      , Lay2mCntrl         = 0x9000 // LAY2M_CNTRL [128] 1000-1fff Layer 2 Message control blocks                        
      , Lay2sCntrl         = 0xA000 // LAY2S_CNTRL[64] 2000-27ff Layer 2 SAP control blocks                            
   };


   enum pfbCommandEnum // definitions related to pfbCommand 
   {
      cmdOFF            = 0xe0        // PFB  - card offline ready to take commands                       
    , cmdON             = 0xe1        // PFB  - card is online ready to take offline command              
    , cmdCOM_CFG        = 0xe2        // PFB  - card is being configured from serial port                 
    , cmdERROR          = 0xef        // PFB  - card is in error, status contains error code              
    , cmdGO_ON          = 0x01        // HOST - card should now go on line                                
    , cmdGO_OFF         = 0x02        // HOST - card should now go off line, or abort config              
    , cmdREINIT         = 0x03        // HOST - card should reinitialize memory and all parameters        
    , cmdCLR_CFG_BUF    = 0x04        // HOST - card should clear pfbBinCfgPage                           
    , cmdCHK_NET_CFG    = 0x05        // HOST - card should check network parameters, and assign defaults 
    , cmdCPY_MAS_CFG    = 0x06        // HOST - card should copy a page of cfg from pfbBinCfgPage         
    , cmdAUTO_BAUD_DET  = 0x07        // HOST - card should do an automatic baud detect                   
    , cmdMAS_ASIGN_ADDR = 0x08        // HOST - assign and fill in data addr (page/offset) for dp master  
    , cmdAUTODP_CFG     = 0x09            // HOST - configure dp master using on-line auto-configuration          
    , cmdCFG_ABF_SHRAM  = 0x0f        // HOST - configure dp master from ABF   binary previously copied   
    , cmdCFG_2BF_SHRAM  = 0x10        // HOST - configure dp master from ET200 binary previously copied   
    , cmdCFG_FROM_FLASH = 0x11        // HOST - configure card from configuration in flash                
    , cmdCFG_FLASH_GO_ON= 0x14        // HOST - configure card from flash then on line                    
    , cmdPGM_TO_FLASH   = 0x21        // HOST - burn card configuration into flash                        
    , cmdERR_ACK        =  0xff        // HOST - acknowledge offline error (online errors cannot be acked) 
   };

   enum pfbMasCntrlCfgEnum  // definitions related to 
   {
      pfbMAS_CTRL_SYNC_SCAN     = 0x01 // enable syncronized scan                                          
    , pfbMAS_CTRL_RUN_MODE      = 0x02 // scan i/o in RUN mode                                             
    , pfbMAS_CTRL_USR_OFS       = 0x04 // user defined rx and tx data offsets (no pfb auto assign)         
    , pfbMAS_CTRL_ENABLE        = 0x08 // enable master mode on PFB card                                   
    , pfbMAS_CTRL_DIS_LED       = 0x10 // disable status led for dp master function                        
    , pfbMAS_CTRL_HOLD_INTR     = 0x20 // hold dp master event interrupt(s) until end of scan              
    , pfbMAS_CTRL_EVT_SCAN_DONE = 0x40 // generate event (intr) at end of scan                             
    , pfbMAS_CTRL_ADDR_ASIGNED  = 0x80 // dp master data addresses have been assigned                      
   };

 
   enum MAS_CNTRL // DP Master control structure 
   {
        masStn         = 0x00 // uchr  station address of slave                                          
                                           
      , masCntCfg      = 0x01 // uchr  Control and configuration register                                
      , masStatus      = 0x02 // uchr  Status Register (Host only Reads)                                 
      , masError       = 0x03 // uchr  Error indication                                                  
      , masEvent       = 0x04 // uchr  Event Flags                                                       
      , masDiagEvent   = 0x05 // uchr  Diagnostic Event Flags                                            
                                           
      , masParmLen     = 0x06 // uchr  Parameters to slave length in bytes                               
      , masChkLen      = 0x07 // uchr  Configuration check to slave length in bytes                      
                                           
      , masRxDataOfs   = 0x08 // ushrt Data received from slave offset within memory page                
      , masTxDataOfs   = 0x0a // ushrt Data to be sent to slave offset within memory page                
                                           
      , masRxDataLen   = 0x0c // uchr  Data received from slave length in bytes                          
      , masTxDataLen   = 0x0d // uchr  Data to be sent to slave length in bytes                          
                                           
      , masSiemType    = 0x0e // uchr  Siemens Device Type                                               
                                           
      , masExtErrInfo  = 0x0f // uchr  Extended error info                                               
                                           
      , masDesig       = 0x10 //-char[13]  1c slave designation (text)                                       
                                           
      , Res2           = 0x1d // uchr  Reserved, DO NOT USE                                              
                                           
      , masDiagMaxLen  = 0x1e // uchr  Maximum length of diag status response                            
      , masDiagLen     = 0x1f // uchr  Diagnostic from slave length in bytes                             
                                           
      , masDiagSts1    = 0x20 // uchr  status byte 1 from slave                                          
      , masDiagSts2    = 0x21 // uchr  status byte 2 from slave                                          
      , masDiagSts3    = 0x22 // uchr  status byte 3 from slave                                          
      , masDiagMasStn  = 0x23 // uchr  Station that configured Slave                                     
      , masDiagID_hi   = 0x24 // uchr  ID hi byte sent back from slave                                   
      , masDiagID_lo   = 0x25 // uchr  ID lo byte sent back from slave                                   
      , masDiagData    = 0x26 //-uchr[24]  3d vendor defined diagnostic info from slave                      
      , masDiagExtOfs  = 0x3e // ushrt offset within extension area if greater than 26 bytes             
                                           
      , masParmMasSts  = 0x40 // uchr  status byte to slave                                              
      , masParmWdFact1 = 0x41 // uchr  Watchdog factor 1 to slave                                        
      , masParmWdFact2 = 0x42 // uchr  Watchdog factor 2 to slave                                        
      , masParmRdyTme  = 0x43 // uchr  Response delay time (tbit) to slave                               
      , masParmID_hi   = 0x44 // uchr  ID value to slave hi                                              
      , masParmID_lo   = 0x45 // uchr  ID value to slave lo                                              
      , masParmGrpId   = 0x46 // uchr  Group ID value for slave (not supported, always 0)                
      , masParmData    = 0x47 //-uchr[23]  5d parameters to slave                                            
      , masParmExtOfs  = 0x5e // ushrt offset within extension area if greater than 25 bytes             
                                           
      , masChkData     = 0x60 //-uchr[30]  7d Configuration check values to slave                            
      , masChkDataExtOfs = 0x7e // ushrt offset within extension area if greater than 32 bytes             
      , masSizeOfTable = 0x80
   };  // end MAS_CNTRL;

   void executeCommand (const unsigned char inCmd);

   unsigned long pageOffset (const unsigned long inNumber) const
      {return (inNumber << 14);}

   unsigned char inputByte (const unsigned long inOffset) const
      {return theMemory -> readChar (theInputsOffset+inOffset);}

   void outputByte ( const unsigned long inOffset
                   , const unsigned char inByte )
      {theMemory -> writeChar (theOutputsOffset+inOffset, inByte);}


   unsigned long theBaseAddress;
   unsigned char theIrq;
   PhysicalMemory * theMemory;
   unsigned long theInputsOffset;
   unsigned long theOutputsOffset;
   unsigned long theMasterControlOffset;
   unsigned char theBaudRate;

};  // end SST5136PFBPCI class declaration

//-----------------------------------------------------------------------------
inline
unsigned char SST5136PFBPCI::state() const
{
   return theMemory -> readChar (pfbCommand);
}

//-----------------------------------------------------------------------------
inline
unsigned char SST5136PFBPCI::status() const
{
   return theMemory -> readChar (pfbStatus);
}

//-----------------------------------------------------------------------------
inline
unsigned char SST5136PFBPCI::baudRate() const
{
   return theMemory -> readChar (pfbBaud);
}


};  // end namespace Uniio

#endif
